@@ -1,5 +1,5 @@
 /* Utility and Unix shadow routines for GNU Emacs support programs on NT.
-   Copyright (C) 1994, 2002-2013 Free Software Foundation, Inc.
+   Copyright (C) 1994, 2002-2015 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -16,31 +16,33 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 
-
 #include <pwd.h>
 #include <malloc.h>
 
 /* Include these headers now so we don't have to worry about include
    order dependencies in common source files.  */
 #include <direct.h>
+#include <fcntl.h>
 #include <io.h>
 #include <stdio.h>
 
 #ifdef sleep
 #undef sleep
 #endif
-void sleep (unsigned long seconds);
+unsigned sleep (unsigned seconds);
 char *getwd (char *dir);
 int getppid (void);
 char * getlogin (void);
 char * cuserid (char * s);
 unsigned getuid (void);
+unsigned geteuid (void);
 unsigned getegid (void);
 unsigned getgid (void);
 int setuid (unsigned uid);
 int setregid (unsigned rgid, unsigned gid);
 char * getpass (const char * prompt);
 int fchown (int fd, unsigned uid, unsigned gid);
+int mkostemp (char * template, int flags);
 
 /* redirect or undo interceptions created by config.h */
 #undef access
@@ -61,15 +63,12 @@ int fchown (int fd, unsigned uid, unsigned gid);
 #undef fopen
 #undef mkdir
 #define mkdir   _mkdir
-#undef mktemp
-#define mktemp  _mktemp
 #undef open
 #define open    _open
 #undef pipe
 #define pipe    _pipe
 #undef read
 #define read    _read
-#undef rename
 #undef rmdir
 #define rmdir   _rmdir
 #undef unlink
