@@ -1,6 +1,6 @@
 ;;; snmp-mode.el --- SNMP & SNMPv2 MIB major mode
 
-;; Copyright (C) 1995, 1998, 2001-2013 Free Software Foundation, Inc.
+;; Copyright (C) 1995, 1998, 2001-2015 Free Software Foundation, Inc.
 
 ;; Author: Paul D. Smith <psmith@BayNetworks.com>
 ;; Keywords: data
@@ -85,8 +85,9 @@
 ;;; Code:
 
 (eval-when-compile
-  (require 'imenu)   ; Need this stuff when compiling for imenu macros, etc.
-  (require 'tempo))
+  (require 'imenu))   ; Need this stuff when compiling for imenu macros, etc.
+
+(require 'tempo)
 
 ;;;----------------------------------------------------------------------------
 ;;
@@ -540,8 +541,6 @@ lines for the purposes of this function."
 ;;
 ;;;----------------------------------------------------------------------------
 
-(require 'tempo)
-
 ;; Perform a completing-read with info given
 ;;
 (defun snmp-completing-read (prompt table &optional pred require init hist)
