@@ -1,6 +1,6 @@
 ;;; pgg-pgp5.el --- PGP 5.* support for PGG.
 
-;; Copyright (C) 1999-2000, 2002-2013 Free Software Foundation, Inc.
+;; Copyright (C) 1999-2000, 2002-2015 Free Software Foundation, Inc.
 
 ;; Author: Daiki Ueno <ueno@unixuser.org>
 ;; Created: 1999/11/02
@@ -26,8 +26,9 @@
 ;;; Code:
 
 (eval-when-compile
-  (require 'cl)				; for pgg macros
-  (require 'pgg))
+  (require 'cl))
+
+(require 'pgg)
 
 (defgroup pgg-pgp5 ()
   "PGP 5.* interface."
