@@ -1,6 +1,6 @@
 ;;; x-win.el --- parse relevant switches and set up for X  -*-coding: iso-2022-7bit;-*-
 
-;; Copyright (C) 1993-1994, 2001-2012 Free Software Foundation, Inc.
+;; Copyright (C) 1993-1994, 2001-2015 Free Software Foundation, Inc.
 
 ;; Author: FSF
 ;; Keywords: terminals, i18n
@@ -67,6 +67,8 @@
 ;; An alist of X options and the function which handles them.  See
 ;; ../startup.el.
 
+(eval-when-compile (require 'cl-lib))
+
 (if (not (fboundp 'x-create-frame))
     (error "%s: Loading x-win.el but not compiled for X" (invocation-name)))
 
@@ -85,7 +87,7 @@
 (defvar x-session-id)
 (defvar x-session-previous-id)
 
-(defun x-handle-no-bitmap-icon (switch)
+(defun x-handle-no-bitmap-icon (_switch)
   (setq default-frame-alist (cons '(icon-type) default-frame-alist)))
 
 ;; Handle the --parent-id option.
@@ -116,13 +118,11 @@ See also `emacs-session-save'.")
 
 (defun emacs-session-filename (session-id)
   "Construct a filename to save the session in based on SESSION-ID.
-If the directory ~/.emacs.d exists, we make a filename in there, otherwise
-a file in the home directory."
-  (let ((basename (concat "session." session-id))
-	(emacs-dir user-emacs-directory))
-    (expand-file-name (if (file-directory-p emacs-dir)
-			  (concat emacs-dir basename)
-			(concat "~/.emacs-" basename)))))
+Return a filename in `user-emacs-directory', unless the session file
+already exists in the home directory."
+  (let ((basename (concat "session." session-id)))
+    (locate-user-emacs-file basename
+                            (concat ".emacs-" basename))))
 
 (defun emacs-session-save ()
   "This function is called when the window system is shutting down.
@@ -425,7 +425,9 @@ as returned by `x-server-vendor'."
 	(#x3fe . ?,D~(B)
 	;; Kana: Fixme: needs conversion to Japanese charset -- seems
 	;; to require jisx0213, for which the Unicode translation
-	;; isn't clear.
+	;; isn't clear.  Using Emacs to convert this to Unicode and back changes
+	;; this from "(J~(B" (i.e., bytes "ESC ( J ~ ESC ( B") to "$(G"#(B" (i.e., bytes
+	;; "ESC $ ( G " # ESC ( B").
 	(#x47e . ?(J~(B)
 	(#x4a1 . ?$A!#(B)
 	(#x4a2 . ?\$A!8(B)
@@ -1125,6 +1127,9 @@ as returned by `x-server-vendor'."
 	(#x20a8 . ?$,1tH(B)
 	(#x20aa . ?$,1tJ(B)
 	(#x20ab . ?$,1tK(B)
+	;; Kana: Fixme: needs checking.  Using Emacs to convert this to Unicode
+	;; and back changes this from ",b$(B" (i.e., bytes "ESC , b $ ESC ( B") to
+	;; ",F$(B" (i.e., bytes "ESC , F $ ESC ( B").
 	(#x20ac . ?,b$(B)))
   (puthash (car pair) (cdr pair) x-keysym-table))
 
@@ -1212,6 +1217,8 @@ The value nil is the same as the list (UTF8_STRING COMPOUND_TEXT STRING)."
 	(remove-text-properties 0 (length text) '(foreign-selection nil) text))
     text))
 
+(defvar x-select-enable-clipboard)	; common-win
+
 ;; Return the value of the current X selection.
 ;; Consult the selection.  Treat empty strings as if they were unset.
 ;; If this function is called twice and finds the same text,
@@ -1305,17 +1312,22 @@ Request data types in the order specified by `x-select-request-type'."
 (declare-function accelerate-menu "xmenu.c" (&optional frame) t)
 
 (defun x-menu-bar-open (&optional frame)
-  "Open the menu bar if `menu-bar-mode' is on, otherwise call `tmm-menubar'."
+  "Open the menu bar if it is shown.
+`popup-menu' is used if it is off."
   (interactive "i")
-  (if (and menu-bar-mode
-	   (fboundp 'accelerate-menu))
-      (accelerate-menu frame)
-    (tmm-menubar)))
+  (cond
+   ((and (not (zerop (or (frame-parameter nil 'menu-bar-lines) 0)))
+	 (fboundp 'accelerate-menu))
+    (accelerate-menu frame))
+   (t
+    (popup-menu (mouse-menu-bar-map) last-nonmenu-event))))
 
 
 ;;; Window system initialization.
 
 (defun x-win-suspend-error ()
+  "Report an error when a suspend is attempted.
+This returns an error if any Emacs frames are X frames, or always under W32."
   ;; Don't allow suspending if any of the frames are X frames.
   (if (memq 'x (mapcar 'window-system (frame-list)))
       (error "Cannot suspend Emacs while running under X")))
@@ -1333,8 +1345,10 @@ Request data types in the order specified by `x-select-request-type'."
 (defvar x-display-name)
 (defvar x-command-line-resources)
 
-(defun x-initialize-window-system ()
+(defun x-initialize-window-system (&optional display)
   "Initialize Emacs for X frames and open the first connection to an X server."
+  (cl-assert (not x-initialized))
+
   ;; Make sure we have a valid resource name.
   (or (stringp x-resource-name)
       (let (i)
@@ -1345,7 +1359,7 @@ Request data types in the order specified by `x-select-request-type'."
 	(while (setq i (string-match "[.*]" x-resource-name))
 	  (aset x-resource-name i ?-))))
 
-  (x-open-connection (or x-display-name
+  (x-open-connection (or display
 			 (setq x-display-name (or (getenv "DISPLAY" (selected-frame))
 						  (getenv "DISPLAY"))))
 		     x-command-line-resources
@@ -1445,8 +1459,10 @@ Request data types in the order specified by `x-select-request-type'."
   ;; 			:help "Paste (yank) text most recently cut/copied")
   ;; 	    nil))
 
+  (x-apply-session-resources)
   (setq x-initialized t))
 
+(add-to-list 'display-format-alist '("\\`[^:]*:[0-9]+\\(\\.[0-9]+\\)?\\'" . x))
 (add-to-list 'handle-args-function-alist '(x . x-handle-args))
 (add-to-list 'frame-creation-function-alist '(x . x-create-frame-with-faces))
 (add-to-list 'window-system-initialization-alist '(x . x-initialize-window-system))
@@ -1581,6 +1597,8 @@ This uses `icon-map-list' to map icon file names to stock icon names."
 	     (and value (cdr value))))
 	 x-gtk-stock-cache))))
 
+(global-set-key [XF86WakeUp] 'ignore)
+
 (provide 'x-win)
 
 ;;; x-win.el ends here
