@@ -1,6 +1,6 @@
 ;;; vc-dir.el --- Directory status display under VC  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2007-2014 Free Software Foundation, Inc.
+;; Copyright (C) 2007-2015 Free Software Foundation, Inc.
 
 ;; Author:   Dan Nicolaescu <dann@ics.uci.edu>
 ;; Keywords: vc tools
@@ -433,7 +433,8 @@ If NOINSERT, ignore elements on ENTRIES which are not in the ewoc."
 	      ;; previous node was in a different directory.
 	      (let* ((rd (file-relative-name entrydir))
 		     (prev-node (ewoc-prev vc-ewoc node))
-		     (prev-dir (vc-dir-node-directory prev-node)))
+		     (prev-dir (if prev-node
+				   (vc-dir-node-directory prev-node))))
 		(unless (string-equal entrydir prev-dir)
 		  (ewoc-enter-before
 		   vc-ewoc node (vc-dir-create-fileinfo rd nil nil nil entrydir))))
