@@ -1,5 +1,5 @@
 /* Indentation functions.
-   Copyright (C) 1985-1988, 1993-1995, 1998, 2000-2014 Free Software
+   Copyright (C) 1985-1988, 1993-1995, 1998, 2000-2015 Free Software
    Foundation, Inc.
 
 This file is part of GNU Emacs.
@@ -2137,10 +2137,15 @@ whether or not it is currently displayed in some window.  */)
 	      if (nlines > 1)
 		move_it_by_lines (&it, min (PTRDIFF_MAX, nlines - 1));
 	    }
-	  else
+	  else	/* it_start = ZV */
 	    {
 	      it.vpos = 0;
 	      move_it_by_lines (&it, min (PTRDIFF_MAX, nlines));
+	      /* We could have some display or overlay string at ZV,
+		 in which case it.vpos will be nonzero now, while
+		 actually we didn't move vertically at all.  */
+	      if (IT_CHARPOS (it) == CHARPOS (pt) && CHARPOS (pt) == it_start)
+		it.vpos = 0;
 	    }
 	}
 
