@@ -1,6 +1,6 @@
 /* 16-bit Windows Selection processing for emacs on MS-Windows
 
-Copyright (C) 1996-1997, 2001-2014 Free Software Foundation, Inc.
+Copyright (C) 1996-1997, 2001-2015 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -600,7 +600,7 @@ DEFUN ("w16-get-clipboard-data", Fw16_get_clipboard_data, Sw16_get_clipboard_dat
       coding.mode |= CODING_MODE_LAST_BLOCK;
       /* We explicitly disable composition handling because selection
 	 data should not contain any composition sequence.  */
-      coding.mode &= CODING_ANNOTATION_MASK;
+      coding.common_flags &= ~CODING_ANNOTATION_MASK;
       decode_coding_object (&coding, Qnil, 0, 0, truelen, truelen, Qt);
       ret = coding.dst_object;
       Vlast_coding_system_used = CODING_ID_NAME (coding.id);
